use std::{
	fs::{File, OpenOptions},
	thread,
	time::Duration,
};

use app::{App, AppState};
use color_eyre::{eyre::eyre, Result};
use gamepad_manager::GamepadManager;
use tracing_subscriber::{layer::SubscriberExt, Layer};

mod app;
mod gamepad_manager;

fn main() -> Result<()> {
	color_eyre::install()?;

	init_tracing();

	let mut manager = GamepadManager::new()?;
	manager.scan_gamepads();
	// if manager.connected_gamepads == 0 {
	// 	return Err(eyre!("No gamepads found!"));
	// }

	let terminal = ratatui::init();
	let app = App::new(manager);

	let app_result = app.run(terminal);
	ratatui::restore();

	app_result
}

#[tracing::instrument]
fn init_tracing() {
	// Create logs folder if it doesn't exist
	let logs_path = std::path::Path::new("logs/");
	if !logs_path.exists() {
		std::fs::create_dir("logs/").expect("Could not create a logs folder at startup!");
	}

	let log_file = OpenOptions::new()
		.write(true)
		.create(true)
		.truncate(true)
		.open("logs/latest.log")
		.unwrap();

	let sub = tracing_subscriber::registry().with(
		tracing_subscriber::fmt::layer()
			.compact()
			.with_ansi(false)
			.with_target(false)
			.with_file(true)
			.with_line_number(true)
			.with_writer(log_file)
			.with_filter(tracing_subscriber::filter::LevelFilter::from_level(
				tracing::Level::INFO,
			))
			.with_filter(tracing_subscriber::filter::EnvFilter::from_default_env()),
	);

	tracing::subscriber::set_global_default(sub).unwrap();
}